<?php

/*
  Plugin Name: Accelerated Mobile Pages Speakeasy
  Plugin URI: http://www.speakeasymarketinginc.com/
  Description: AMP for WP - Accelerated Mobile Pages for WordPress
  Version: 1.0
  Author: Muhammad Yousaf Raza, and Speakeasy Team
 */

// Exit if accessed directly.
if (!defined('ABSPATH'))
    exit;

if (!class_exists("AMPMYR")) {

    class AMPMYR {

        var $_AMPMYR_VERSION = "1.0";

        function __construct() {
            define('AMPMYR_PLUGIN_DIR', plugin_dir_path(__FILE__));
            define('AMPMYR_IMAGE_DIR', plugin_dir_url(__FILE__) . 'images');
            // Load Files required for the plugin to run
            require AMPMYR_PLUGIN_DIR . 'includes/register-menu.php';
            /* ------------------------------------------- */
            register_activation_hook(__FILE__, array($this, 'ampmyr_activate'));
            register_deactivation_hook(__FILE__, array($this, 'ampmyr_rewrite_deactivate'));
            add_action('init', array($this, 'AmpQVar'));
            add_action('init', array($this, 'ampmyr_is_amp_endpoint'));
            add_action('init', array($this, 'ampmyr_init'));
        }

        public function AmpQVar() {
            define('AMP_QUERY_VAR', apply_filters('amp_query_var', 'amp'));
            add_rewrite_endpoint(AMP_QUERY_VAR, EP_ALL);
        }

        public function ampmyr_activate() {
            flush_rewrite_rules();
        }

        public function ampmyr_rewrite_deactivate() {
            global $wp_rewrite;
            foreach ($wp_rewrite->endpoints as $index => $endpoint) {
                if (AMP_QUERY_VAR === $endpoint[1]) {
                    unset($wp_rewrite->endpoints[$index]);
                    break;
                }
            }
            flush_rewrite_rules();
        }

        public function ampmyr_init() {
            add_action('wp', array($this, 'ampmyr_add_actions'));
        }

        public function ampmyr_add_actions() {
            if ($this->ampmyr_is_amp_endpoint()) {
                $this->ampmyr_prepare_render();
            } else {
                add_action('wp_head', array($this, 'ampmyr_frontend_add_canonical'));
            }
        }

        public function ampmyr_frontend_add_canonical() {
            $amp_url = $this->ampmyr_get_permalink(get_queried_object_id());
            printf('<link rel="amphtml" href="%s" />', esc_url($amp_url));
        }

        public function ampmyr_get_permalink($post_id) {
            if ('' != get_option('permalink_structure')) {
                $amp_url = trailingslashit(get_permalink($post_id)) . user_trailingslashit(AMP_QUERY_VAR, 'single_amp');
            } else {
                $amp_url = add_query_arg(AMP_QUERY_VAR, 1, get_permalink($post_id));
            }

            return apply_filters('ampmyr_get_permalink', $amp_url, $post_id);
        }

        public function ampmyr_prepare_render() {
            add_action('template_redirect', array($this, 'ampmyr_render'));
        }

        public function ampmyr_render() {
            $P_ID = $this->GetPageID();
            require AMPMYR_PLUGIN_DIR . 'includes/template-class.php';
            require AMPMYR_PLUGIN_DIR . 'includes/helper.php';
            /* ............................................. */
            $AMP_Temp_Obj = new AMPMYR_Templates($P_ID);
            $AMP_Temp_Obj->Load_Head();
            do_action('ampmyr_post_template_header');
            $AMP_Temp_Obj->Load_Body();
            $AMP_Temp_Obj->Load_Footer();
            do_action('ampmyr_post_template_footer');

            exit;
        }

        public function GetPageID() {
            if (is_home()) {
                return get_option('page_on_front');
            } else {
                return get_queried_object_id();
            }
        }

        public function ampmyr_is_amp_endpoint() {
            return false !== get_query_var('amp', false);
        }

        public function PrinTArray($Array) {
            echo "<pre>" . print_r($Array, true) . "</pre>";
        }

    }

    $AMPMYR_Obj = new AMPMYR();
}